@@ -67,10 +67,8 @@ struct dvd_copy {
 
 int main(int argc, char **argv) {
 
-	/**
-	 * Parse options
-	 */
-
+	bool debug = false;
+	char device_filename[PATH_MAX];
 	bool opt_track_number = false;
 	bool opt_chapter_number = false;
 	bool opt_cell_number = false;
@@ -97,6 +95,7 @@ int main(int argc, char **argv) {
 		{ "track", required_argument, 0, 't' },
 		{ "help", no_argument, 0, 'h' },
 		{ "version", no_argument, 0, 'V' },
+		{ "debug", no_argument, 0, 'z' },
 		{ 0, 0, 0, 0 }
 
 	};
@@ -113,7 +112,7 @@ int main(int argc, char **argv) {
 	memset(dvd_copy.buffer, '\0', DVD_VIDEO_LB_LEN);
 	memset(dvd_copy.filename, '\0', PATH_MAX);
 
-	while((opt = getopt_long(argc, argv, "c:d:ho:t:V", long_options, &long_index )) != -1) {
+	while((opt = getopt_long(argc, argv, "c:d:ho:t:Vz", long_options, &long_index )) != -1) {
 
 		switch(opt) {
 
@@ -229,6 +228,10 @@ int main(int argc, char **argv) {
 				printf("dvd_copy %s\n", PACKAGE_VERSION);
 				return 0;
 
+			case 'z':
+				debug = true;
+				break;
+
 			// ignore unknown arguments
 			case '?':
 				invalid_opt = true;
@@ -264,10 +267,11 @@ int main(int argc, char **argv) {
 		arg_last_chapter = 1;
 	}
 
-	const char *device_filename = DEFAULT_DVD_DEVICE;
-
+	memset(device_filename, '\0', PATH_MAX);
 	if (argv[optind])
-		device_filename = argv[optind];
+		strncpy(device_filename, argv[optind], PATH_MAX - 1);
+	else
+		strncpy(device_filename, DEFAULT_DVD_DEVICE, PATH_MAX - 1);
 
 	/** Begin dvd_copy :) */
 
@@ -437,7 +441,7 @@ int main(int argc, char **argv) {
 	dvdread_vts_file = DVDOpenFile(dvdread_dvd, vts, DVD_READ_TITLE_VOBS);
 
 	if(p_dvd_copy)
-		printf("Track: %*" PRIu16 ", Length: %s, Chapters: %*" PRIu8 ", Cells: %*" PRIu8 ", Audio streams: %*" PRIu8 ", Subpictures: %*" PRIu8 ", Filesize: %.0lf MBs\n", 2, dvd_track.track, dvd_track.length, 2, dvd_track.chapters, 2, dvd_track.cells, 2, dvd_track.audio_tracks, 2, dvd_track.subtitles, dvd_track.filesize_mbs);
+		printf("Track: %*" PRIu16 ", Length: %s, Chapters: %*" PRIu8 ", Cells: %*" PRIu8 ", Audio streams: %*" PRIu8 ", Subpictures: %*" PRIu8 ", Title set: %*" PRIu16 ", Filesize: %.0lf MBs\n", 2, dvd_track.track, dvd_track.length, 2, dvd_track.chapters, 2, dvd_track.cells, 2, dvd_track.audio_tracks, 2, dvd_track.subtitles, 2, vts, dvd_track.filesize_mbs);
 
 	// Check for track issues
 	dvd_track.valid = true;
@@ -503,6 +507,7 @@ int main(int argc, char **argv) {
 	uint64_t cell_sectors = 0;
 	uint64_t cell_block = 0;
 	ssize_t cell_blocks_read = 0;
+	ssize_t total_blocks_read = 0;
 	ssize_t bytes_written = 0;
 	ssize_t total_bytes_written = 0;
 
@@ -532,7 +537,7 @@ int main(int argc, char **argv) {
 			cell_sectors = dvd_cell.last_sector - dvd_cell.first_sector;
 
 			if(p_dvd_copy)
-				printf("        Chapter: %*" PRIu8 ", Cell: %" PRIu8 ", VTS: %" PRIu16 ", Filesize: % 5.0lf MBs\n", 2, dvd_chapter.chapter, 2, dvd_cell.cell, vts, ceil(dvd_cell.filesize / 1048576.0));
+				printf("        Chapter: %*" PRIu8 ", Cell: %*" PRIu8 ", Filesize: % 5.0lf MBs\n", 2, dvd_chapter.chapter, 2, dvd_cell.cell, ceil(dvd_cell.filesize / 1048576.0));
 
 			if(dvd_cell.last_sector > dvd_cell.first_sector)
 				cell_sectors++;
@@ -540,9 +545,10 @@ int main(int argc, char **argv) {
 			cell_block = dvd_cell.first_sector;
 
 			// This is where you would change the boundaries -- are you dumping to a track file (no boundaries) or a VOB (boundaries)
-			while(cell_block < dvd_cell.last_sector) {
+			while(cell_block < dvd_cell.last_sector + 1) {
 
 				cell_blocks_read = DVDReadBlocks(dvdread_vts_file, (size_t)cell_block, 1, dvd_copy.buffer);
+				total_blocks_read += cell_blocks_read;
 
 				// Zero out cells that couldn't be read
 				if(cell_blocks_read == -1)
@@ -560,7 +566,14 @@ int main(int argc, char **argv) {
 						percent_complete = 99.0;
 				}
 
-				fprintf(stderr, "Progress: %.0lf/%.0lf MBs (%.0lf%%)\r", mbs_written, dvd_copy.filesize_mbs, percent_complete);
+				// FIXME causes bleeding on previous lines
+				/*
+				if(debug)
+					fprintf(stderr, "Progress: %.0lf/%.0lf MBs (%.0lf%%)  Blocks: %li/%li\r", mbs_written, dvd_copy.filesize_mbs, percent_complete, total_blocks_read, dvd_copy.blocks);
+				else
+				*/
+					fprintf(stderr, "Progress: %.0lf/%.0lf MBs (%.0lf%%)\r", mbs_written, dvd_copy.filesize_mbs, percent_complete);
+
 				fflush(stderr);
 
 				cell_block++;
