@@ -25,7 +25,10 @@
 #include "dvd_init.h"
 #ifdef __linux__
 #include <linux/cdrom.h>
+#include <linux/limits.h>
 #include "dvd_drive.h"
+#else
+#include <limits.h>
 #endif
 
 	/**
@@ -57,6 +60,7 @@ int main(int argc, char **argv) {
 	bool d_chapters = false;
 	bool d_subtitles = false;
 	bool d_cells = false;
+	bool d_title_sets = false;
 
 	// How much output
 	bool debug = false;
@@ -64,6 +68,10 @@ int main(int argc, char **argv) {
 	// limit results
 	bool d_has_audio = false;
 	bool d_has_subtitles = false;
+	bool d_has_alang = false;
+	bool d_has_slang = false;
+	char d_alang[DVD_AUDIO_LANG_CODE + 1] = {'\0'};
+	char d_slang[DVD_AUDIO_LANG_CODE + 1] = {'\0'};
 	bool d_longest = false;
 	bool opt_min_seconds = true;
 	unsigned long int arg_number = 0;
@@ -87,7 +95,7 @@ int main(int argc, char **argv) {
 	uint8_t d_stream_num = 0;
 
 	// Device hardware
-	const char *device_filename = NULL;
+	char device_filename[PATH_MAX];
 
 	// libdvdread
 	dvd_reader_t *dvdread_dvd = NULL;
@@ -98,7 +106,7 @@ int main(int argc, char **argv) {
 	struct dvd_info dvd_info;
 
 	// Video Title Set
-	struct dvd_vts dvd_vts[99];
+	struct dvd_vts dvd_vts[DVD_MAX_VTS_IFOS];
 
 	// Display formats
 	const char *display_formats[4] = { "Pan and Scan or Letterbox", "Pan and Scan", "Letterbox", "Unset" };
@@ -110,7 +118,7 @@ int main(int argc, char **argv) {
 	int ix = 0;
 	int opt = 0;
 	bool invalid_opt = false;
-	const char p_short_opts[] = "aAcdE:ghijlLM:sST:t:uVvxz";
+	const char p_short_opts[] = "aAcdeE:gG:hijlLM:N:sST:t:uVvxz";
 	struct option p_long_opts[] = {
 
 		{ "track", required_argument, NULL, 't' },
@@ -121,6 +129,7 @@ int main(int argc, char **argv) {
 		{ "chapters", no_argument, NULL, 'c' },
 		{ "subtitles", no_argument, NULL, 's' },
 		{ "cells", no_argument, NULL, 'd' },
+		{ "title-sets", no_argument, NULL, 'e' },
 		{ "all", no_argument, NULL, 'x' },
 
 		{ "json", no_argument, NULL, 'j' },
@@ -132,7 +141,9 @@ int main(int argc, char **argv) {
 		{ "min-seconds", required_argument, NULL, 'E' },
 		{ "min-minutes", required_argument, NULL, 'M' },
 		{ "has-audio", no_argument, NULL, 'A' },
+		{ "has-alang", required_argument, NULL, 'N' },
 		{ "has-subtitles", no_argument, NULL, 'S' },
+		{ "has-slang", required_argument, NULL, 'G' },
 		{ "valid", no_argument, NULL, 'L' },
 
 		{ "help", no_argument, NULL, 'h' },
@@ -165,6 +176,10 @@ int main(int argc, char **argv) {
 				d_cells = true;
 				break;
 
+			case 'e':
+				d_title_sets = true;
+				break;
+
 			case 'E':
 				opt_min_seconds = true;
 				arg_min_seconds = (uint32_t)strtoul(optarg, NULL, 10);
@@ -176,6 +191,15 @@ int main(int argc, char **argv) {
 				d_chapters = true;
 				break;
 
+			case 'G':
+				if(strlen(optarg) != 2 || !isalpha(optarg[0]) || !isalpha(optarg[1])) {
+					fprintf(stderr, "Subtitle language code must be two characters\n");
+					return 1;
+				}
+				strncpy(d_slang, optarg, DVD_AUDIO_LANG_CODE);
+				d_has_slang = true;
+				break;
+
 			case 'i':
 				p_dvd_id = true;
 				break;
@@ -203,6 +227,15 @@ int main(int argc, char **argv) {
 				arg_min_minutes = (uint32_t)strtoul(optarg, NULL, 10);
 				break;
 
+			case 'N':
+				if(strlen(optarg) != 2 || !isalpha(optarg[0]) || !isalpha(optarg[1])) {
+					fprintf(stderr, "Audio language code must be two characters\n");
+					return 1;
+				}
+				strncpy(d_alang, optarg, DVD_AUDIO_LANG_CODE);
+				d_has_alang = true;
+				break;
+
 			case 's':
 				d_subtitles = true;
 				break;
@@ -274,12 +307,15 @@ int main(int argc, char **argv) {
 				printf("  -s, --subtitles       Display VobSub subtitles\n");
 				printf("  -c, --chapters        Display chapters\n");
 				printf("  -d, --cells           Display cells\n");
+				printf("  -e, --title-sets      Display video title sets\n");
 				printf("  -x, --all             Display all\n");
 				printf("\n");
 				printf("Narrow results:\n");
 				printf("  -l, --longest		Track with the longest length\n");
 				printf("  -A, --has-audio       Track has audio streams\n");
+				printf("  -N, --has-alang <xx>  Track has audio audio language, two character code\n");
 				printf("  -S, --has-subtitles   Track has VobSub subtitles\n");
+				printf("  -G, --has-slang <xx>  Track has subtitle language, two character code\n");
 				printf("  -E, --seconds <secs>  Track has minimum number of seconds\n");
 				printf("  -M, --minutes <mins>  Track has minimum number of minutes\n");
 				printf("  -T, --vts <number>    Track is in video title set number\n");
@@ -308,10 +344,11 @@ int main(int argc, char **argv) {
 	// If '-i /dev/device' is not passed, then set it to the string
 	// passed.  fex: 'dvd_info /dev/dvd1' would change it from the default
 	// of '/dev/dvd'.
+	memset(device_filename, '\0', PATH_MAX);
 	if (argv[optind])
-		device_filename = argv[optind];
+		strncpy(device_filename, argv[optind], PATH_MAX - 1);
 	else
-		device_filename = DEFAULT_DVD_DEVICE;
+		strncpy(device_filename, DEFAULT_DVD_DEVICE, PATH_MAX - 1);
 
 	// Exit after all invalid input warnings have been sent
 	if(valid_args == false)
@@ -328,7 +365,7 @@ int main(int argc, char **argv) {
 	// Open VMG IFO -- where all the cool stuff is
 	vmg_ifo = ifoOpen(dvdread_dvd, 0);
 	if(vmg_ifo == NULL || !ifo_is_vmg(vmg_ifo)) {
-		fprintf(stderr, "Opening VMG IFO failed\n");
+		fprintf(stderr, "Opening VMG IFO failed, can not open DVD\n");
 		DVDClose(dvdread_dvd);
 		return 1;
 	}
@@ -364,17 +401,11 @@ int main(int argc, char **argv) {
 		d_last_track = dvd_info.tracks;
 	}
 
-	ifo_handle_t *vts_ifos[DVD_MAX_VTS_IFOS];
-
 	if(opt_vts && (arg_vts == 0 || arg_vts > dvd_info.video_title_sets)) {
 		fprintf(stderr, "Video Title Set must be between 1 and %" PRIu16 "\n", dvd_info.video_title_sets);
 		return 1;
 	}
 
-	uint8_t vts_ifo_ix;
-	for(vts_ifo_ix = 0; vts_ifo_ix < 100; vts_ifo_ix++)
-		vts_ifos[vts_ifo_ix] = NULL;
-
 	// Do some checks to see if a VTS is ok or not
 	for(vts = 1; vts < dvd_info.video_title_sets + 1; vts++)
 		dvd_vts[vts] = dvd_vts_open(dvdread_dvd, vts);
@@ -424,7 +455,7 @@ int main(int argc, char **argv) {
 	}
 
 	// Count valid, invalid tracks and title sets
-	if(debug || opt_vts) {
+	if(debug || opt_vts || d_title_sets) {
 
 		for(ix = 1; ix <= dvd_info.tracks; ix++) {
 			dvd_vts[dvd_tracks[ix].vts].tracks++;
@@ -440,7 +471,7 @@ int main(int argc, char **argv) {
 	}
 
 	// Print the valid and invalid VTSs
-	if(debug) {
+	if(debug || d_title_sets) {
 
 		printf("        Tracks: %*" PRIu16 ", ", 2, dvd_info.tracks);
 		printf("Valid: %*" PRIu16 ", ", 2, dvd_info.valid_tracks);
@@ -514,9 +545,38 @@ int main(int argc, char **argv) {
 			continue;
 
 		// Skip if limiting to valid only
-		if(d_is_valid && dvd_track.valid == false)
+		if(d_is_valid && (dvd_vts[dvd_track.vts].valid == false || dvd_track.valid == false))
 			continue;
 
+		// Need an IFO if checking for audio or sub track languages
+		if(d_has_alang || d_has_slang) {
+			if(dvd_vts[dvd_track.vts].valid == false) {
+				fprintf(stderr, "VTS %" PRIu16 " is invalid, skipping\n", dvd_track.vts);
+			} else {
+				vts_ifo = ifoOpen(dvdread_dvd, dvd_track.vts);
+			}
+		}
+
+		// Skip if audio track language stream isn't found
+		if(d_has_alang && !dvd_track_has_audio_lang_code(vts_ifo, d_alang)) {
+			ifoClose(vts_ifo);
+			vts_ifo = NULL;
+			continue;
+		}
+
+		// Skip if subtitle language stream isn't found
+		if(d_has_slang && !dvd_track_has_subtitle_lang_code(vts_ifo, d_slang)) {
+			ifoClose(vts_ifo);
+			vts_ifo = NULL;
+			continue;
+		}
+
+		// Close out for next round
+		if(vts_ifo) {
+			ifoClose(vts_ifo);
+			vts_ifo = NULL;
+		}
+
 		// Display track information
 		printf("Track: %*" PRIu16 ", ", 2, dvd_track.track);
 		printf("Length: %s, ", dvd_track.length);
